@@ -1,14 +1,14 @@
 package Class::PObject;
 
-# $Id: PObject.pm,v 1.39.2.10 2003/09/06 14:03:06 sherzodr Exp $
+# $Id: PObject.pm,v 1.43 2003/09/09 08:46:33 sherzodr Exp $
 
 use strict;
 #use diagnostics;
 use Log::Agent;
 use vars ('$VERSION', '$revision');
 
-$VERSION    = '2.06_03';
-($revision) = '$Revision: 1.39.2.10 $' =~ m/Revision:\s*(\S+)/;
+$VERSION    = '2.07_01';
+($revision) = '$Revision: 1.43 $' =~ m/Revision:\s*(\S+)/;
 
 # configuring Log::Agent
 logconfig(-level=>$ENV{POBJECT_DEBUG} || 0);
@@ -104,19 +104,26 @@ sub pobject {
     ${ "$class\::props" }   = $props;
 
     require Class::PObject::Template;
+    # To ensure operator overloading works properly on these
+    # objects, let's also set the caller's @ISA array:
+    push @{ "$class\::ISA" }, "Class::PObject::Template";
 
-    *{ "$class\::new" }         = \&Class::PObject::Template::new;
-    *{ "$class\::columns" }     = \&Class::PObject::Template::columns;
-    *{ "$class\::load" }        = \&Class::PObject::Template::load;
-    *{ "$class\::fetch" }       = \&Class::PObject::Template::fetch;
-    *{ "$class\::save" }        = \&Class::PObject::Template::save;
-    *{ "$class\::remove" }      = \&Class::PObject::Template::remove;
-    *{ "$class\::remove_all" }  = \&Class::PObject::Template::remove_all;
-    *{ "$class\::count" }       = \&Class::PObject::Template::count;
-    *{ "$class\::errstr" }      = \&Class::PObject::Template::errstr;
-    *{ "$class\::dump" }        = \&Class::PObject::Template::dump;
-    *{ "$class\::__props" }     = \&Class::PObject::Template::__props;
-    *{ "$class\::__driver" }    = \&Class::PObject::Template::__driver;
+    ####
+    # After dealing with ISA, I don't think the following would be
+    # necessary.
+
+    #*{ "$class\::new" }         = \&Class::PObject::Template::new;
+    #*{ "$class\::columns" }     = \&Class::PObject::Template::columns;
+    #*{ "$class\::load" }        = \&Class::PObject::Template::load;
+    #*{ "$class\::fetch" }       = \&Class::PObject::Template::fetch;
+    #*{ "$class\::save" }        = \&Class::PObject::Template::save;
+    #*{ "$class\::remove" }      = \&Class::PObject::Template::remove;
+    #*{ "$class\::remove_all" }  = \&Class::PObject::Template::remove_all;
+    #*{ "$class\::count" }       = \&Class::PObject::Template::count;
+    #*{ "$class\::errstr" }      = \&Class::PObject::Template::errstr;
+    #*{ "$class\::dump" }        = \&Class::PObject::Template::dump;
+    #*{ "$class\::__props" }     = \&Class::PObject::Template::__props;
+    #*{ "$class\::__driver" }    = \&Class::PObject::Template::__driver;
 
     for my $colname ( @{ $props->{columns} } ) {
         if ( $class->UNIVERSAL::can($colname) ) {
@@ -132,6 +139,7 @@ sub pobject {
             return $get->( $_[0], $colname )
         }
     }
+
 }
 
 logtrc 1, "%s loaded successfully", __PACKAGE__;
@@ -146,7 +154,7 @@ Class::PObject - Simple framework for programming persistent objects
 
 =head1 SYNOPSIS
 
-After loading the Class::PObject with C<use>, we can declare a class like so
+After loading Class::PObject with C<use>, we can declare a class:
 
     pobject Person => {
         columns     => ['id', 'name', 'email'],
@@ -158,16 +166,16 @@ We can also declare the class in its own F<.pm> file:
     package Person;
     use Class::PObject;
     pobject {
-        columns     => ['id', 'name', 'email''
+        columns     => ['id', 'name', 'email'],
         datasource  => './data'
     };
 
-We can now create an instance of above Person, and fill it in with data, and
-store it into disk:
+We can now create an instance of above Person, fill it with data, and
+save it:
 
     $person = new Person();
     $person->name('Sherzod');
-    $person->email('sherzodr@cpan.org');
+    $person->email('sherzodr[AT]cpan.org');
     $new_id = $person->save()
 
 We can access the saved Person later, make necessary changes and save back:
@@ -197,17 +205,17 @@ We can also seek into a specific point of the result set:
 =head1 DESCRIPTION
 
 Class::PObject is a simple class framework for programming persistent objects in Perl.
-Such objects can store themselves into disk, and recreate themselves from disk.
+Such objects can store themselves into disk, and restore themselves from disk.
 
 =head1 PHILOSOPHY
 
-This section has been moved into L<Class::PObject::Philosophy|Class::PObject::Philosophy>.
+This section has been moved to L<Class::PObject::Philosophy|Class::PObject::Philosophy>.
 
 =head1 PROGRAMMING STYLE
 
-The style of Class::PObject is very similar to that of L<Class::Struct|Class::Struct>.
-Instead of exporting 'struct()', however,  Class::PObject exports C<pobject()> function.
-Another visual difference is the way you declare the class. In Class::PObject, each property
+Programming style of Class::PObject is very similar to that of L<Class::Struct|Class::Struct>.
+Instead of exporting C<struct()>, however,  Class::PObject exports C<pobject()> function.
+Another visual difference is the way you declare classes. In Class::PObject, each property
 of the class is represented as a I<column>.
 
 Suppose, you have a database called "person" with the following records:
@@ -221,29 +229,28 @@ Suppose, you have a database called "person" with the following records:
 
 =head2 CLASS DECLARATIONS
 
-Let's declare a class first to represent the above data as a Persistent Object (pobject for short).
-To do this, we first load the Class::PObject with C<use>, and declare a class with C<pobject()>
-function, like so:
+Let's declare a class to represent the above data as a Persistent Object (pobject for short).
+To do this, we first load Class::PObject with C<use> and declare a class with C<pobject()>
+construct:
 
     use Class::PObject;
     pobject Person => {
         columns => ["id", "name", "email"]
     };
 
-Above construct is declaring a Class representing a Person object. Person object
-has 3 attributes that are called I<columns> in the I<pobject()> declaration. These are I<id>,
-I<name> and I<email>.
+Above construct is declaring a class representing a Person object. Person object
+has 3 attributes - I<id>, I<name> and I<email>.
 
-Above is called in-line declaration, because you are creating an inline object -
-the one that doesn't need to be in its own class file. You could declare it almost anywhere
-inside your Perl code.
+Above is called in-line declaration, because you are creating an inline class -
+the one that doesn't need to be in its own F<.pm> file. You can declare inline classes
+almost anywhere inside your Perl code.
 
 In-line declarations are not very useful, because you cannot access them separately
 from within another application without having to re-declare identical class several times
 in each of your programs.
 
 Another, more recommended way of declaring classes is in their own F<.pm> files. For example,
-inside a F<Person.pm> file we may put:
+inside a F<Person.pm> file we say:
 
     # lib/Person.pm
     package Person;
@@ -254,8 +261,6 @@ inside a F<Person.pm> file we may put:
 
     __END__;
 
-That can be the whole content of your Perl module.
-
 Now, from any other application all we need to do is to load F<Person.pm>, and access all
 the nifty things it has to offer:
 
@@ -267,8 +272,8 @@ the nifty things it has to offer:
 
 From the above class declaration you may be wondering, how does it now how and where the object data
 are stored? The fact is, it doesn't. That's why by default it stores your objects in your system's temporary
-folder, wherever it may be, using default L<file|Class::PObject::Driver::file> driver. To control this
-behavior you can define I<driver> and I<datasource> attributes in addition to the above I<columns> attribute:
+folder - wherever it may be - using default L<file|Class::PObject::Driver::file> driver. To control this
+behavior you can define I<driver> and/or I<datasource> attributes in addition to above I<columns>:
 
     pobject Person => {
         columns     => ["id", "name", "email"],
@@ -281,15 +286,15 @@ the objects in your custom, F<./data> folder.
 You could've also chosen to store your objects in a DBM file, or in mysql tables. That's where
 you will need to define your I<driver> attribute.
 
-To store them in L<BerkelyDB|BerkelyDB>, using L<DB_File|DB_File>
+To store them in I<DBM> files using L<DB_File|DB_File>:
 
     pobject Person => {
-        columns => ["id", "name", "email"],
-        driver  => 'db_file',
-        datasource => './data'
+        columns     => ["id", "name", "email"],
+        driver      => 'db_file',
+        datasource  => './data'
     };
 
-To store them in Comma Separated text files using L<DBD::CSV|DBD::CSV>:
+To store them in I<comma separated> text files using L<DBD::CSV|DBD::CSV>:
 
     pobject Person => {
         columns => ["id", "name", "email"],
@@ -311,7 +316,38 @@ Or, to store them in a mysql database using L<DBD::mysql|DBD::mysql>:
         }
     };
 
-So forth. For more options you should refer to respective object driver manual.
+So forth. For more options you should refer to respective driver manuals. Following
+object drivers are available in this distribution:
+
+=over 4
+
+=item L<file|Class::PObject::Driver::file>
+
+Default driver. Stores objects of the same type in a dedicated folder. Individual
+objects are stored in their own files.
+
+=item L<csv|Class::PObject::Driver::csv>
+
+Stores objects of the same type in a single file. Each object is separated by a new line.
+Object attributes are separated with comma. These files can be easily loaded to your
+spreadsheet applications, such as Excel, Access or even RDBMS tables.
+
+=item L<db_file|Class::PObject::Driver::db_file>
+
+Stores objects of the same type in a single file using L<DB_File|DB_File>.
+
+=item L<sqlite|Class::PObject::Driver::sqlite>
+
+SQLite driver. Stores objects using L<DBD::SQLite|DBD::SQLite>. Depending on the I<datasource>
+attribute, all the objects can be stored in a single file, or on different files.
+
+=item L<mysql|Class::PObject::Driver::mysql>
+
+Stores objects in MySQL tables. Objects of the same type are stored in a single table.
+Individual rows of these tables represent individuals objects. Each column type
+corresponds to object attributes.
+
+=back
 
 =head2 CREATING NEW PERSON
 
@@ -330,47 +366,52 @@ As soon as you call C<save()> method of the C<$person>, all the records will be
 the disk.
 
 Notice, we didn't give any value for the I<id> column. Underlying object drivers
-will automatically generate a new ID for your newly created object, and C<save()>
-method will return this ID for you.
+automatically generate a new ID for newly created objects and C<save()>
+method returns this ID for you.
 
-If you assign a value for I<id>, you better make sure that ID doesn't already exist. If it does,
-the old object with that ID will be replaced with new object. So to be safe, just don't bother
-defining any values for your ID columns, unless you have a really good reason.
+If you assign a value to I<id>, you better make sure that ID doesn't already exist.
+If it does, the old object with that ID will be replaced with new object. To be safe,
+just don't bother defining any values for your ID columns, unless you have a really good reason.
 
-Sometimes, if you have objects with few attributes, you may choose to both create your
-Person and assign its values at the same time. You can do so by passing your column values
+Sometimes, if you have objects with few attributes, you may choose both to create the
+Person and to assign its attributes at the same time. You can do so by passing your column values
 while creating the new person:
 
     $person = new Person(name=>"Sherzod Ruzmetov", email=>"sherzodr[AT]cpan.org");
     $person->save();
 
-=head2 LOADING OBJECTS
+=head2 LOADING A SINGLE OBJECT
 
-PObjects support C<load()> class method, which allows you to retrieve your objects from the disk.
-You can retrieve objects in many ways. The easiest, and the most efficient way of loading an object
-from the disk is by its id:
+PObjects support C<load()> class method, which allows you to retrieve your
+objects from the disk. You can retrieve objects in many ways. The easiest,
+and the most efficient way of loading an object from the disk is by its id:
 
     $person = Person->load(217);
 
-Now, assuming the C<$person> could be retrieved successfully, we can access the attributes of the object like so:
+Now, assuming C<$person> could be retrieved successfully, we can access the
+attributes of the object like so:
 
     printf( "Hello %s!\n", $person->name )
 
 Notice, we are using the same method names to access them as the ones we used to assign values with,
 but this time with no arguments.
 
-Above, instead of displaying the C<$person>'s name, we could also edit the name and save it back:
+Above, instead of displaying C<$person>'s name, we could've also edited his
+name and save it back:
 
     $person->name("Sherzod The Geek");
     $person->save();
 
+=head2 LOADING MULTIPLE OBJECTS
+
 Sometimes you may choose to load multiple objects. Using the same C<load()> method,
 we could assign all the result set into an array:
 
     @people = Person->load();
 
-Each element of the C<@people> is a C<$person> object, and you could list all of them with the following
-syntax:
+Above code is retrieving all the objects from the database, and assigning them to
+C<@people> - array. Each element of C<@people> is a C<$person> object. We can
+list all of them with the following syntax:
 
     for my $person ( @people ) {
         printf("[%d] - %s <%s>\n", $person->id, $person->name, $person->email)
@@ -379,11 +420,14 @@ syntax:
 Notice two different contexts C<load()> was used in. If you call C<load()> in scalar context,
 regardless of the number of matching objects, you will always retrieve the first object in
 the data set. For added efficiency, Class::PObject will add I<limit=E<gt>1> argument to C<load()>
-even if it's missing, or exists with a different value.
+even if it's missing, or exists with a different value. We will talk more about I<attributes>
+very soon.
 
 If you called C<load()> in array context, you will always receive an array of objects, even
 if result set consists of a single object.
 
+=head2 LOADING OBJECTS SELECTIVELY
+
 Sometimes you just want to load objects matching a specific criteria, say, you want all the people
 whose name are I<John>. You can achieve this by passing a hashref as the first argument to C<load()>:
 
@@ -401,16 +445,15 @@ C<load()> in the form of a hashref and are called I<arguments> :
 Above C<@people> holds 100 C<$person> objects, all sorted by name in descending order.
 We could use both terms and arguments at the same time and in any combination.
 
-=head2 SUPPORTED ARGUMENTS OF load()
-
 Arguments are the second set of key/value pairs passed to C<load()>. Some drivers
-may look at this set as post-result-filters.
+may look at this set as post-result filters.
 
 =over 4
 
 =item C<sort>
 
-Defines which column the list should be sorted in.
+Defines which column the list should be sorted by. Value of this attribute
+can be any valid column name this particular class has.
 
 =item C<direction>
 
@@ -433,17 +476,17 @@ value and up to I<limit> number will be returned to the caller.
 =head2 INCREMENTAL LOAD
 
 C<load()> may be all you need most of the time. If your objects are of larger size,
-or if you need to operate on thousands of objects, your program may not have enough memory to
+or if you need to operate on large amount of objects, your program may not have enough memory to
 hold them all, because C<load()> tends to load all the matching objects to the memory.
 
 If this is your concern, you are better off using C<fetch()> method instead. Syntax of C<fetch()>
-is almost identical to C<load()>, with an exception that it doesn't accept object id
+is almost identical to C<load()>, with a single exception: it doesn't accept object id
 as the first argument. You can either use it without any arguments, or with any combination
 of C<\%terms> and C<\%args> as needed, just like with C<load()>.
 
 Another important difference is, it does not return any objects. It's return value is an
-instance of L<Class::PObject::Iterator|Class::PObject::Iterator>, which helps
-you to iterate through large data sets by loading them one at a time inside a C<while>-loop:
+L<iterator object|Class::PObject::Iterator>. Iterator helps you to iterate through large
+data sets by loading them one at a time inside a C<while>-loop:
 
     $result = Person->fetch();
     while ( my $person = $result->next ) {
@@ -455,7 +498,7 @@ you to iterate through large data sets by loading them one at a time inside a C<
         ...
     }
 
-For the list of methods available for C<$result> - iterator object refer to its
+For the list of methods available for iterator object refer to its
 L<manual|Class::PObject::Iterator>.
 
 =head2 COUNTING OBJECTS
@@ -466,27 +509,26 @@ how many people are in your database in total, or how many "John"s are there.
 You can of course do it with a syntax similar to:
 
     @all = People->load();
-    $count = scalar( @all );
+    $count = scalar @all;
 
-This however, also means you will be loading all the objects to memory at the same time.
+This also means you will be loading all the objects to memory at the same time.
 
-Even if we could've done it using an iterator class, as discussed earlier, some database
-engines may provide a more optimized way of retrieving this information without having
-to C<load()> any objects, by consulting available meta information. That's where
-C<count()> class method comes in:
+Even if we could've done it using an L<iterator|Class::PObject::Iterator>, as discussed
+earlier, some database engines may provide more optimized way of retrieving this
+information without having to C<load()> any objects, by consulting available meta information.
+That's where C<count()> class method comes in:
 
     $count = Person->count();
 
-C<count()> can accept \%terms, just like above C<load()> does as the first argument.
-Using \%terms you can define conditions:
+C<count()> can accept C<\%terms> as the first argument, just like above C<load()> does.
+Using C<\%terms> you can define conditions:
 
     $njohns = Person->count({name=>"John"});
 
 =head2 REMOVING OBJECTS
 
 PObjects support C<remove()> and C<remove_all()> methods. C<remove()> is an object method.
-It is used only to remove one object at a time. C<remove_all()> is a class method, which removes
-all the objects of the same type, thus a little more scarier.
+It is used only to remove one object at a time.
 
 To remove a person with id 217, we first need to create an object of that Person, and only
 then call C<remove()> method:
@@ -495,13 +537,13 @@ then call C<remove()> method:
     $person->remove();
 
 C<remove_all()> is a static class method, and is used for removing all the objects from the
-database:
+disk:
 
     Person->remove_all();
 
 C<remove_all()> can also be used for removing objects selectively without having to load them
 first. To do this, you can pass C<\%terms> as the first argument to C<remove_all()>. These C<\%terms>
-are the same as the ones we used for C<load()>:
+are the same as the ones we used for C<load()>, C<fetch()> and C<count()>:
 
     Person->remove_all({rating=>1});
 
@@ -513,27 +555,25 @@ all the objects:
         $person->remove
     }
 
-However, this will require first loading the object to the memory one at a time, and then
-removing one at a time. Most of the object drivers may offer a better, efficient way of removing
+However, this will require loading the object to the memory one at a time, and then
+removing one at a time. Most object drivers may offer a better, efficient way of removing
 objects from the disk without having to C<load()> them. That's why you should rely on
 C<remove_all()>.
 
 =head2 COLUMN TYPES
 
-Class::PObject lets you define types for your columns, also known as I<type-maps>. 
+Class::PObject lets you define types for your columns, also known as I<type-maps>.
 First off, what for?
 
 Type-maps can be looked at as properties of each column. If you are familiar with RDBMS,
 you are already familiar with them. They read as I<CHAR>, I<VARCHAR>, I<INTEGER>,
-I<ENCRYPT>, I<MD5> etc.
+I<TEXT>, I<BLOB> etc.
 
 Column types can be used for declaring a property for a column, defining a constraint
-or input-output filtering of columns.
+or input-output filtering of columns. This same feature can also be used for defining
+object relationships, but let's talk about it later.
 
-This same feature can also be used for defining object relationships, but let's talk
-about it later, not to confuse you yet.
-
-You can define a type for your columns by I<tmap> pobject attribute:
+You can define types for your columns by I<tmap> pobject attribute (I<tmap> stands for I<type map>):
 
     pobject User => {
         columns => ['id', 'login', 'psswd', 'name'],
@@ -551,9 +591,9 @@ and so forth.
 
 You normally never have to declare any column types. If you don't, I<id> column will always
 default to I<INTEGER>, and all other columns will always default to I<VARCHAR(255)>. So
-you should declare your column types only if you don't agree with default types.
+you should declare your column types only if you don't agree with defaults.
 
-So in above example, I could've chosen to say:
+In above example I could've chosen to say:
 
     pobject User => {
         columns => ['id', 'login', 'psswd', 'name'],
@@ -562,14 +602,12 @@ So in above example, I could've chosen to say:
         }
     }
 
-Notice, I am accepting default type values for all the columns except for I<psswd> column,
-which should be encrypted using MD5 message digest algorithm before being stored into
-disk.
+Notice, I am accepting default types for all the columns except for I<psswd> column,
+which should be encrypted using MD5 algorithm before being stored into disk.
 
 Of course, if I didn't care about security as much, I could've chosen not to define
 type-map for I<psswd> column either.
 
-
 As of this release, available built-in column types are:
 
 =over 4
@@ -590,10 +628,9 @@ in future releases, so be cautious.
 
 =item VARCHAR(n)
 
-Stands for VARIABLE CHARacter I<n> character long. Highest allowed value for I<n>
-is usually 255 characters, but this is still not enforced, but this may change
-in future releases, so be cautious. Handled through 
-L<Class::PObject::Type::VARCHAR|Class::PObject::Type::VARCHAR> class.
+Stands for VARIABLE CHARacter I<n> characters long. Highest allowed value for I<n>
+is usually 255 characters, but not yet enforced. This may change in future releases,
+so be cautious. Handled through  L<Class::PObject::Type::VARCHAR|Class::PObject::Type::VARCHAR>.
 
 =item TEXT
 
@@ -603,8 +640,8 @@ This may change in future releases, so be cautious.
 
 =item ENCRYPT
 
-Stands for ENCRYPTed column types. It denotes values which need to be encrypted
-using UNIX's C<crypt()> function before being stored into disk. Smart overloading allows
+Stands for ENCRYPTed. It denotes a value that need to be encrypted
+using UNIX's C<crypt()> before being stored into disk. Smart overloading allows
 to do something like the following:
 
 
@@ -625,13 +662,13 @@ to 'ZG9nqo.9bPjGA' if encrypted.
 
 =item MD5
 
-Stands for MD5 message digest. It denotes values that need to be encrypted
-using MD5 message digest algorithm before being stored into disk. Smart overloading
-allows it to be used in similar way as I<ENCRYPT> column types
+Stands for MD5 - Message Digest algorithm. It denotes value that need to be encrypted
+using MD5 algorithm before being stored into disk. Smart overloading
+allows it to be used in similar way as I<ENCRYPT> column type.
 
 =back
 
-Up to this section you noticed that objects' accessor methods have been returning
+Up to this point you noticed that objects' accessor methods have been returning
 strings, right? Wrong! They have been returning objects of their appropriate types.
 But you didn't notice them and kept treating them as strings because of smart
 operator overloading feature of those objects.
@@ -645,11 +682,12 @@ column, you could simply use Perl's built-in I<ref()> function:
 =head2 DEFINING METHODS OTHER THAN ACCESSORS
 
 In some cases accessor methods are not all the methods your class may ever need. It may
-need some other behaviors. In cases like these, you can extend your class with custom methods.
+need some other behaviors. In cases like these, you can extend your class with custom
+methods.
 
-For example, assume you have a "User" object, which needs to be authenticated
+For example, assume you have a I<User> object, which needs to be authenticated
 before they can access certain parts of the web site. It may be a good idea to
-add C<authenticate()> method into your "User" class, which either returns a User
+add C<authenticate()> method into your I<User> class, which either returns a I<User>
 object if he/she is logged in properly, or returns undef, meaning the user isn't
 logged in yet.
 
@@ -657,6 +695,7 @@ To do this we can simply define additional method, C<authenticate()> inside our
 Consider the following example:
 
     package User;
+    use Class::PObject;
 
     pobject {
         columns     => ['id', 'login', 'psswd', 'email'],
@@ -667,13 +706,18 @@ Consider the following example:
         my $class = shift;
         my ($cgi, $session) = @_;
 
-        # if the user is already logged in, return the object:
+        unless ( $cgi->isa('CGI') && $session->isa('CGI::Session') ) {
+            croak "authenticate(): usage error"
+        }
+
+        # if user's session indicates he/she is logged in, we load the User object
+        # by that id and return it
         if ( my $user_id = $session->param('_logged_in') ) {
             return $class->load( $user_id )
         }
 
         # if we come this far, the user is not logged in yet, but still
-        # might've submitted our login form:
+        # might've submitted the login form:
         my $login     = $cgi->param('login')    or return 0;
         my $password  = $cgi->param('password') or return 0;
 
@@ -693,7 +737,7 @@ Now, we can check if the user is logged into our web site with the following cod
 
     use User;
     my $user = User->authenticate($cgi, $session);
-    unless ( defined $user ) {
+    unless ( $user ) {
         die "You need to login to the web site before you can access this page!"
     }
     printf "<h2>Hello %s</h2>", $user->login;
@@ -703,11 +747,11 @@ You can do it differently depending on the tools you're using.
 
 =head2 HAS-A RELATIONSHIP THROUGH TYPE MAPPING
 
-Some available tools prefer calling this feature as I<HAS-A> relationship, 
-but Class::PObject prefers the term I<type-mapping>. For the sake of familiarity, 
+Some available tools prefer calling this feature as I<HAS-A> relationship,
+but Class::PObject prefers the term I<type-mapping>. For the sake of familiarity,
 let's look at it as I<HAS-A> relationship for now.
 
-I<HAS-A> relationship says that a value in a specific column is actually an id
+I<HAS-A> relationship says that a value in a specific column is an index
 of another object. In RDBMS this particular column is known as I<foreighn key>, because
 it holds the primary key of another (foreign) table.
 
@@ -741,42 +785,37 @@ Now, we can create a new article with the following syntax:
     $article->author( $author );
     $article->save();
 
-Notice, we passed C<$author> object to article's C<author()> method. 
+Notice, we passed C<$author> object to article's C<author()> method.
 Class::PObject retrieves the ID of the author and stores it into Article's
 I<author> field.
 
 When you choose to access I<author> field of the article, Class::PObject
-will automatically load respective Author object.
+will automatically load respective Author object from database.
 
     $article = Article->load(32);
     my $author = $article->author;
     printf "Author: %s\n", $author->name;
 
-
-We could've also passed C<$author> as part of the \%terms while loading
+We could've also passed C<$author> as part of the C<\%terms> while loading
 articles:
 
     my $author = Author->load({name=>"Sherzod Ruzmetov"});
     @my_articles = Article->load({author=>$author});
 
-
 As you noticed, defining HAS-A relationship is the same as defining
-built-in column types. In fact, they are the same, with the exception
-of smart overloading, which are available only in built-in type-maps, 
-and not for objects in HAS-A relationship.
-
-One can look at it as a restriction, but you don't have to :-).
+built-in column types, it's because the engine behind both column types and
+column relationships are handled through the same API.
 
 =head2 ERROR HANDLING
 
-I<PObjects> try never to C<die()>, and lets the programer to decide what to do on failure,
+I<PObject>s try never to C<die()>, and lets the programer to decide what to do on failure,
 (unless of course, you insult it with wrong syntax).
 
-Methods that may fail are the ones to do with disk access, namely, C<save()>, C<load()>, 
-C<remove()> and C<remove_all()>. So it's advised you check these methods' return values before 
-you assume any success. If an error occurs, the above methods return undef. More verbose error 
-message will be accessible through errstr() method. In addition, C<save()> method should 
-always return the object id on success:
+Methods that may fail are the ones to do with disk access, namely, C<save()>, C<load()>,
+C<fetch()>, C<count()>, C<remove()> and C<remove_all()>. So it's advised you check these
+methods' return values before you assume any success. If an error occurs, the above methods
+return undef. More verbose error  message will be accessible through C<errstr()> static
+class method. In addition, C<save()> method should always return the object id on success:
 
     my $new_id = $person->save();
     unless ( defined $new_id ) {
@@ -796,7 +835,7 @@ few useful ones:
 
 C<columns()> - returns hash-reference to all the columns of the object. Keys of the hash
 hold column names, and their values hold respective column values. All the objects will be
-stringified.
+stringified:
 
     my $columns = $person->columns();
     while ( my ($k, $v) = each %$columns ) {
@@ -820,35 +859,75 @@ This error message is also available through C<$CLASS::errstr> global variable:
 
 =item *
 
-C<__props()> - returns I<class properties>. Class properties are usually whatever was 
-passed to C<pobject()> as a hashref. This information is usually useful for driver authors only.
+C<__props()> - returns I<class properties>. Class properties are usually whatever was
+passed to C<pobject()> as a hashref, either as the first argument or the second.
+This information is usually helpful for driver authors, and for those who want to access
+certain information about this class.
 
 =item *
 
-C<__driver()> - returns either already available driver object, or creates a new object and returns it.
-Although not recommended, you can use this driver object to access driver's low-level functionality,
-as long as you know what you are doing. For available driver methods consult with specific driver
-manual, or contact the vendor.
+C<__driver()> - returns either already available driver object, or creates a new object
+and returns it. Although not recommended, you can use this driver object to access
+driver's low-level functionality, as long as you know what you are doing. For available
+driver methods consult with specific driver manual, or contact the vendor.
+
+=item *
+
+C<drop_datasource()> - used when you want to drop storage device allocated for storing
+this particular object. Its meaning varies depending on the pobject driver being used.
+L<file|Class::PObject::Driver::file> driver may remove the entire directory of objects, 
+whereas L<mysql|Class::PObject::Driver::mysql> driver may drop the table.
 
 =back
 
 =head1 TODO
 
-Following are the lists of features and/or fixes that need to be applied before considering
-the library ready for production environment. The list is not exhaustive. Feel free to add your
-suggestions.
+Class::PObject is not a tool for solving all the problems of the Universe.
+It may be its success rather than its failure. I still want it to do certain things,
+for I believe they are right down its alley.
 
-=head2 MORE FLEXIBLE load()
+=over 4
 
-C<load()> will not be all we need until it supports at least simple I<join>s.
-I believe it's something to be supported by object drivers, that's where it can be performed more
-efficiently.
+=item * Multiple object joins
 
-=head2 GLOBAL DESCTRUCTOR
+Currently C<load()> and C<fetch()> methods are used for querying a single object. It would
+be ideal if we could select multiple objects at a time through some I<join> syntax.
+
+=item * Column Indexes
+
+While RDBMS drivers do not need this, L<db_file|Class::PObject::Driver::db_file>,
+L<csv|Class::PObject::Driver::csv> and L<file|Class::PObject::Driver::file> drivers
+will not be suitable for production environment until they do.
+
+Column indexes are needed to retrieve more complex queries much faster, without
+having to perform linear search. With aforementioned drivers (especially
+L<file|Class::PObject::Driver::file> and L<csv|Class::PObject::Driver::csv>) once number
+of records increases, querying objects gets extremely processor intensive if more
+complex C<load()> and C<fetch()> methods are used.
+
+=item * SET Column types
+
+Currently column types can be configured to be of I<HAS-A> relationship. It would be
+great if we could specify I<HAS-MANY> relationships. Following syntax is still to be
+implemented:
+
+    pobject Article => {
+        columns => ['id', 'title', 'authors'],
+        tmap    => {
+            authors => ['Author']
+        }
+    };
+
+Notice, I<authors> field is set to be able to hold an array of I<Author> objects.
+
+=back
+
+
+=head1 NOTES
 
 Pobjects try to cache the driver object for more extended periods than pobject's scope permits them
-to. So a I<global desctuctor> should be applied to prevent unfavorable behaviors, especially 
-under persistent environments, such as mod_perl or GUI.
+to. So a I<global destructor> should be applied to prevent unfavorable behaviors, especially
+under persistent environments, such as mod_perl
 
 Global variables that I<may> need to be cleaned up are:
 
@@ -859,9 +938,9 @@ Global variables that I<may> need to be cleaned up are:
 Where C<$drivername> is the name of the driver used. If more than one driver is used in your project,
 more of these variables may exist. This variable holds particular driver object.
 
-=item B<$PObjectName::props>
+=item B<$ClassName::props>
 
-Holds the properties for this particular PObject named C<$PObjectName>. For example, if you created
+Holds the properties for this particular PObject named C<$ClassName>. For example, if you created
 a pobject called I<Person>, then it's properties are stored in global variable C<$Person::props>.
 
 =back
@@ -875,17 +954,39 @@ application we could've done something like:
 
 =head1 DRIVER SPECIFICATIONS
 
-L<Class::PObject::Driver>, L<Class::PObject::Driver::DBI>
+You can learn more about pobject driver specs in the following manuals:
+
+=over 4
+
+=item L<Class::PObject::Driver|Class::PObject::Driver>
+
+Base pobject driver specification. All the drivers are expected to inherit from
+this class.
+
+=item L<Class::PObject::Driver::DBI|Class::PObject::Driver::DBI>
+
+Base pobject driver for all the DBI-related drivers. This inherits from
+Class::PObject::Driver, but re-defines certain methods with those more relevant
+to DBI drivers.
+
+=item L<Class::PObject::Driver::DBM|Class::PObject::Driver::DBM>
+
+Base pobject driver for all the DBM-related drivers. It inherits from
+Class::PObject::Driver, and re-defines certain methods with those more relevant
+to DBM
+
+=back
+
+=head1 COLUMN TYPE SPECIFICATIONS
+
+You can learn more about column type specs from L<Class::PObject::Type|Class::PObject::Type>,
+which is also a base class of all the types.
 
 =head1 SEE ALSO
 
-L<Class::PObject>,
-L<Class::PObject::Driver>,
-L<Class::PObject::Driver::file>,
-L<Class::PObject::Driver::DBI>,
-L<Class::PObject::Driver::csv>,
-L<Class::PObject::Driver::mysql>,
-L<Class::PObject::Driver::file>
+http://poop.sourceforge.net/, although little dated, provides brief overview
+of available tools for programming Persistent Objects in Perl. As of this writing,
+Class::PObject is still not mentioned.
 
 =head1 AUTHOR
 
@@ -898,6 +999,6 @@ Copyright (c) 2003 Sherzod B. Ruzmetov. All rights reserved.
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
 
-$Date: 2003/09/06 14:03:06 $
+$Date: 2003/09/09 08:46:33 $
 
 =cut
