@@ -1,5 +1,7 @@
 package Class::PObject::Driver::db_file;
 
+# $Id: db_file.pm,v 1.4 2003/09/09 08:46:36 sherzodr Exp $
+
 use strict;
 #use diagnostics;
 use vars ('$VERSION', '@ISA');
@@ -28,6 +30,24 @@ sub dbh {
 
 
 
+sub drop_datasource {
+    my ($self, $object_name, $props) = @_;
+
+    my (undef, $dbh, $unlock) = $self->dbh($object_name, $props, 'w') or return;
+    my $filename = $self->_filename($object_name, $props);
+    unless ( unlink $filename ) {
+        $self->errstr( "couldn't unlink '$filename': $!" );
+        return undef
+    }
+    $unlock->();
+
+    return 1
+}
+
+
+
+
+
 sub _filename {
     my ($self, $object_name, $props) = @_;
 
